"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const extend = require("deep-extend");
const fs = require("fs");
const path = require("path");
const writePackage = require("write-pkg");
const util_1 = require("../util");
const cache_1 = require("./cache");
const conf_1 = require("./conf");
const logger_1 = require("./logger");
const render_1 = require("./render");
const { resolve } = path;
const { RdTypes } = conf_1.default;
exports.default = {
    async checkEnv() {
        await util_1.default.asyncExec('docker -v');
        await util_1.default.asyncExec('docker-compose -v');
        // whether is running or not
        await util_1.default.asyncExec('docker info');
    },
    async pull(image) {
        logger_1.spinner.start(`Pulling ${image} from docker...`);
        await util_1.default.asyncExec(`docker pull ${image}`);
        logger_1.spinner.stop();
    },
    async copy(image, mappings) {
        logger_1.spinner.start(`Copy files from ${image}...`);
        if (await this.containerExist('rde-dummy')) {
            await util_1.default.asyncExec('docker container rm /rde-dummy');
        }
        await util_1.default.asyncExec(`docker create -it --name rde-dummy ${image} bash`);
        for (const mapping of mappings) {
            await util_1.default.asyncExec(`docker cp rde-dummy:${mapping.from} ${mapping.to}`);
        }
        await util_1.default.asyncExec('docker rm -fv rde-dummy');
        logger_1.spinner.stop();
    },
    async imageExist(name) {
        try {
            await util_1.default.asyncExec(`docker inspect --type=image ${name}`);
            return true;
        }
        catch (e) {
            if (e) { }
            return false;
        }
    },
    async containerExist(name) {
        try {
            await util_1.default.asyncExec(`docker ps -a | grep ${name}`);
            return true;
        }
        catch (e) {
            if (e) { }
            return false;
        }
    },
    async tag(name) {
        await util_1.default.asyncExec(`docker tag -a | grep ${name}`);
    },
    async push(name) {
        await util_1.default.asyncExec(`docker push ${name}`);
    },
    async genDockerFile(workDir, from, dir) {
        // gen dockerfile
        await render_1.default.renderTo('docker/.dockerignore', {}, `${dir}/.dockerignore`, {
            overwrite: true,
        });
        await render_1.default.renderTo('docker/Dockerfile', {
            from,
            workDir,
            dockerWorkDirRoot: conf_1.default.dockerWorkDirRoot,
        }, `${dir}/Dockerfile`, {
            overwrite: true,
        });
    },
    async genDockerCompose(workDir, cmd, ports, watch, tag, dir) {
        if (!tag) {
            const name = path.basename(conf_1.default.cwd);
            tag = `${name}:latest`;
        }
        await render_1.default.renderTo('docker/docker-compose', {
            workDir,
            cmd,
            ports,
            watch,
            tag,
        }, `${dir}/docker-compose.yml`, {
            overwrite: true,
        });
    },
    /**
     * rde will regenerate package.json under such condition:
     * 1. no cache info found
     * 2. rdc version changed if rdType is application
     * 3. any package.json under template dir has changed if rdType is container
     * 4. extends attr in rdc.config.js has changed if rdType is container
     */
    async genPkgJson() {
        const { rdType, runtimeDir, cwd, localCacheDir, rdcConfName, } = conf_1.default;
        const nodeModulesDir = resolve(runtimeDir, 'node_modules');
        const isApplication = rdType === RdTypes.Application;
        let rdcName;
        if (isApplication) {
            const rdaConf = conf_1.default.getAppConf();
            rdcName = rdaConf.container.name || '';
        }
        const isContainer = rdType === RdTypes.Container;
        let pkgJsonLMTs;
        let extendsName;
        let pkgJson = null;
        if (isContainer) {
            // last-modified-time
            const pkgPath = resolve(cwd, 'template/package.json');
            pkgJsonLMTs = fs.statSync(pkgPath).mtimeMs;
            pkgJson = require(pkgPath);
            const rdcConfPath = resolve(cwd, rdcConfName);
            const rdcConf = require(rdcConfPath);
            extendsName = rdcConf.extends;
        }
        if (!cache_1.default.exist ||
            !fs.existsSync(nodeModulesDir) ||
            (isApplication && rdcName !== cache_1.default.get('rda.container')) ||
            (isContainer && pkgJsonLMTs !== cache_1.default.get('rdc.pkg.lmts')) ||
            (isContainer && extendsName && extendsName !== cache_1.default.get('rdc.extends'))) {
            const image = isApplication ? rdcName : extendsName;
            const resultPkgJson = await this.getPkgJson(image, pkgJson);
            await writePackage(`${localCacheDir}`, resultPkgJson);
            if (isApplication) {
                cache_1.default.set('rda.container', rdcName);
            }
            if (isContainer) {
                cache_1.default.set('rdc.pkg.lmts', pkgJsonLMTs);
                extendsName && cache_1.default.set('rdc.extends', extendsName);
            }
        }
    },
    async getPkgJson(image, result = {}) {
        if (!image) {
            return result;
        }
        const { dockerWorkDirRoot, localCacheDir, rdcConfName, } = conf_1.default;
        const name = image.split(':')[0];
        const srcDir = `${dockerWorkDirRoot}/${name}`;
        const destPkgPath = `${localCacheDir}/package.json`;
        const destRdcConfPath = `${localCacheDir}/${rdcConfName}`;
        await this.copy(image, [{
                from: `${srcDir}/package.json`,
                to: destPkgPath,
            }, {
                from: `${srcDir}/${rdcConfName}`,
                to: destRdcConfPath,
            }]);
        const rdcConf = require(destRdcConfPath);
        let pkgJson = {};
        if (fs.existsSync(destPkgPath)) {
            pkgJson = require(destPkgPath);
        }
        pkgJson = extend(pkgJson, result);
        await util_1.default.asyncExec(`rm ${destPkgPath} ${destRdcConfPath}`);
        if (!rdcConf.extends) {
            return pkgJson;
        }
        await this.getPkgJson(rdcConf.extends, pkgJson);
    }
};
